@@ -13,7 +13,7 @@ use crate::response::Response;
 use crate::error;
 use crate::body::{Body, NonStreamingBody};
 use serde::{Serialize, Deserialize, Deserializer};
-use serde::de::{MapAccess};
+use serde::de::{Error, MapAccess};
 use serde::ser::SerializeMap;
 use serde_json::Value;
 use crate::headers::{AddHeaders, SortedHeaders};
@@ -106,7 +106,11 @@ impl std::hash::Hash for Request {
     fn hash<H: std::hash::Hasher>(&self, state: &mut H) {
         self.method().hash(state);
         self.url().hash(state);
-        self.headers().iter().for_each(|(k, v)| {
+        let mut sorted = self.headers().iter()
+            .map(|(k, v)| (k.as_str(), v.as_bytes()))
+            .collect::<Vec<(&str, &[u8])>>();
+        sorted.sort();
+        sorted.into_iter().for_each(|(k, v)| {
             k.hash(state);
             v.hash(state);
         });
@@ -270,6 +274,22 @@ impl<'a> RequestBuilder<'a> {
         self
     }
 
+    pub fn header(mut self, key: &str, value: &str) -> Self {
+        self.headers.insert(
+            HeaderName::from_str(key).unwrap(),
+            HeaderValue::from_str(value).unwrap()
+        );
+        self
+    }
+
+    pub fn bearer_auth(mut self, token: &str) -> Self {
+        self.headers.insert(
+            hyper::header::AUTHORIZATION,
+            hyper::header::HeaderValue::from_str(&format!("Bearer {}", token)).unwrap()
+        );
+        self
+    }
+
     pub fn json<S: Serialize>(mut self, obj: S) -> Self {
         self.body = Some(Body::Json(serde_json::to_value(obj).unwrap()));
         self.headers.insert(
@@ -304,6 +324,8 @@ impl<'a> RequestBuilder<'a> {
                 let v = match v {
                     Value::String(s) => Cow::Borrowed(s.as_ref()),
                     Value::Number(n) => Cow::Owned(n.to_string()),
+                    Value::Bool(b) => Cow::Owned(b.to_string()),
+                    Value::Null => Cow::Borrowed(""),
                     _ => panic!("Invalid query value"),
                 };
                 let v = urlencoding::encode(&v);
@@ -368,7 +390,9 @@ impl<'a> RequestBuilder<'a> {
 
 #[cfg(test)]
 mod tests {
+    use std::collections::hash_map::DefaultHasher;
     use std::collections::HashMap;
+    use std::hash::{Hash, Hasher};
     use super::*;
     use http::Method;
 
@@ -401,14 +425,27 @@ mod tests {
         let r1 = Request(hyper::Request::builder()
             .method(Method::POST)
             .header("content-type", "application/json")
+            .header("user-agent", "httpclient/0.1.0")
             .uri("http://example.com/")
             .body(Body::Json(serde_json::to_value(&data).unwrap())).unwrap());
         let r2 = Request(hyper::Request::builder()
             .method(Method::POST)
+            .header("user-agent", "httpclient/0.1.0")
             .header("content-type", "application/json")
             .uri("http://example.com/")
             .body(Body::Json(serde_json::to_value(&data).unwrap())).unwrap());
         assert_eq!(r1, r2);
+        let h1 = {
+            let mut s = DefaultHasher::new();
+            r1.hash(&mut s);
+            s.finish()
+        };
+        let h2 = {
+            let mut s = DefaultHasher::new();
+            r2.hash(&mut s);
+            s.finish()
+        };
+        assert_eq!(h1, h2);
     }
 
     #[test]
@@ -425,7 +462,7 @@ mod tests {
     fn test_query() {
         let client = Client::new(None);
         let r1 = RequestBuilder::new(&client, Method::GET, "http://example.com/foo/bar".parse().unwrap())
-            .query(HashMap::from([("a", "b"), ("c", "d")]));
-        assert_eq!(r1.uri.to_string(), "http://example.com/foo/bar?a=b&c=d");
+            .query(HashMap::from([("a", Some("b")), ("c", Some("d")), ("e", None)]));
+        assert_eq!(r1.uri.to_string(), "http://example.com/foo/bar?a=b&c=d&e=");
     }
 }
\ No newline at end of file
