@@ -9,12 +9,9 @@ use crate::body::{Body, NonStreamingBody};
 use serde::{Serialize, Deserialize, Deserializer};
 use serde::ser::SerializeMap;
 
-
-
-use serde::de::{MapAccess};
+use serde::de::{Error, MapAccess};
 use crate::headers::{AddHeaders, SortedHeaders};
 
-
 #[derive(Debug, Serialize)]
 #[serde(transparent)]
 pub struct Response(
@@ -57,6 +54,15 @@ impl Response {
         }
     }
 
+    pub fn error_for_status_ref(&self) -> Result<&Self, &Self> {
+        let status = self.status();
+        if status.is_server_error() || status.is_client_error() {
+            Err(&self)
+        } else {
+            Ok(&self)
+        }
+    }
+
     pub async fn text(mut self) -> Result<String, crate::Error> {
         let bytes = hyper::body::to_bytes(self.0.body_mut()).await?;
         let encoding = Encoding::for_label(&[]).unwrap_or(encoding_rs::UTF_8);
@@ -173,4 +179,4 @@ impl<'de> Deserialize<'de> for Response {
     {
         deserializer.deserialize_map(ResponseVisitor)
     }
-}
\ No newline at end of file
+}
